using Microsoft.AspNetCore.Mvc;
using response;
using context;
using helper;
using models;
using request;

[ApiController]
[Route("api/[controller]")]
public class AssignmentController : ControllerBase {
    private readonly ILogger<AssignmentController> _logger;
    private readonly TestPlatformResponse _response;
    public AssignmentHelper _helper { get ; set ; }
    public AssignmentController(ILogger<AssignmentController> logger, TestPlatformContext context) {
        TestPlatformResponse response = new TestPlatformResponse();
        _helper = new AssignmentHelper(response, context);
        _logger = logger;
        _response = response;
    }
    [HttpGet]
    public IActionResult assignments() {
        try{
            return Ok(_helper.getAllAssignments());
        }catch(Exception){
            _response.error = "unhandled exception.";
            _response.data = null;
            _response.status = Status.Failed;
            return NotFound(_response);
        }
    }
    [HttpPost]
    public IActionResult newAssignment(AssignmentRequest assignment) {
        try{
            return Ok(_helper.newAssignment(assignment));
        }catch(Exception){
            _response.error = "unhandled exception.";
            _response.data = null;
            _response.status = Status.Failed;
            return NotFound(_response);
        }
    }
    [HttpDelete("{id}")]
    public IActionResult deleteAssignment(int id) {
        try{
            return Ok(_helper.deleteAssignment(id));
        }catch(Exception){
            _response.error = "unhandled exception.";
            _response.data = null;
            _response.status = Status.Failed;
            return NotFound(_response);
        }
    }
    [HttpPut("{id}")]
    public IActionResult updateAssignment(int id, AssignmentRequest assignmentRequest){
        try{
            return Ok(_helper.updateAssignment(id, assignmentRequest));
        }catch(Exception){
            _response.error = "unhandled exception.";
            _response.data = null;
            _response.status = Status.Failed;
            return NotFound(_response);
        }

    }
    [HttpGet("{id}")]
    public IActionResult assignment(int id) {
        try{
            return Ok(_helper.getSingleAssignment(id));
        }catch(Exception){
            _response.error = "unhandled exception.";
            _response.data = null;
            _response.status = Status.Failed;
            return NotFound(_response);
        }
    }
}