using Microsoft.AspNetCore.Mvc;
using Hangfire;
using response;
using context;
using helper;
using models;
using request;
using hubs;


[ApiController]
[Route("api/[controller]")]
public class AttemptController : ControllerBase
{
    private readonly ILogger<AttemptController> _logger;
    private readonly TestPlatformResponse _response;
    public AttemptHelper _helper { get; set; }
    public AttemptController(ILogger<AttemptController> logger, TestPlatformContext context,
     IBackgroundJobClient backgroundJobClient) {
        TestPlatformResponse response = new TestPlatformResponse();
        _helper = new AttemptHelper(response, context, backgroundJobClient);
        _logger = logger;
        _response = response;
    }
    [HttpPost]
    public IActionResult newAttempt(Attempt attempt)
    {
        try
        {
            return Ok(_helper.newAttempt(attempt, ""));
        }
        catch (Exception e)
        {
            _response.error = "unhandled exception." + e.StackTrace;
            _response.data = null;
            _response.status = Status.Failed;
            return NotFound(_response);
        }
    }
    [HttpDelete("{id}")]
    public IActionResult deleteAttempt(int id)
    {
        try
        {
            return Ok(_helper.deleteAttempt(id));
        }
        catch (Exception)
        {
            _response.error = "unhandled exception.";
            _response.data = null;
            _response.status = Status.Failed;
            return NotFound(_response);
        }
    }
    [HttpGet("{id}")]
    public IActionResult attempt(int id)
    {
        try
        {
            return Ok(_helper.getSingleAttempt(id));
        }
        catch (Exception)
        {
            _response.error = "unhandled exception.";
            _response.data = null;
            _response.status = Status.Failed;
            return NotFound(_response);
        }
    }
}