using Microsoft.AspNetCore.Mvc;
using request;
using models;
using response;
using context;
using helper;

[ApiController]
[Route("api/[controller]")]
public class LanguageController : ControllerBase {
    private readonly ILogger<LanguageController> _logger;
    private readonly TestPlatformResponse _response;
    public LanguageHelper _helper { get ; set ; }
    public LanguageController(ILogger<LanguageController> logger, TestPlatformContext context) {
        TestPlatformResponse response = new TestPlatformResponse();
        _helper = new LanguageHelper(response, context);
        _logger = logger;
        _response = response;
    }
    [HttpGet]
    public IActionResult languages() {
        try{
            return Ok(_helper.getAllLanguages());
        }catch(Exception){
            _response.error = "unhandled exception.";
            _response.data = null;
            _response.status = Status.Failed;
            return NotFound(_response);
        }
    }
    [HttpPost]
    public IActionResult newLanguage(Language language) {
        try{
            return Ok(_helper.newLanguage(language));
        }catch(Exception){
            _response.error = "unhandled exception.";
            _response.data = null;
            _response.status = Status.Failed;
            return NotFound(_response);
        }
    }
    [HttpDelete("{id}")]
    public IActionResult deleteLanguage(int id) {
        try{
            return Ok(_helper.deleteLanguage(id));
        }catch(Exception){
            _response.error = "unhandled exception.";
            _response.data = null;
            _response.status = Status.Failed;
            return NotFound(_response);
        }
    }
    [HttpGet("{id}")]
    public IActionResult language(int id) {
        try{
            return Ok(_helper.getSingleLanguage(id));
        }catch(Exception){
            _response.error = "unhandled exception.";
            _response.data = null;
            _response.status = Status.Failed;
            return NotFound(_response);
        }
    }
}