using Microsoft.AspNetCore.Mvc;
using request;
using models;
using response;
using context;
using helper;

[ApiController]
[Route("api/[controller]")]
public class LanguageVersionController : ControllerBase {
    private readonly ILogger<LanguageVersionController> _logger;
    private readonly TestPlatformResponse _response;
    public LanguageVersionHelper _helper { get ; set ; }
    public LanguageVersionController(ILogger<LanguageVersionController> logger, TestPlatformContext context) {
        TestPlatformResponse response = new TestPlatformResponse();
        _helper = new LanguageVersionHelper(response, context);
        _logger = logger;
        _response = response;
    }
    [HttpGet]
    public IActionResult languageVersions() {
        try{
            return Ok(_helper.getAllLanguageVersions());
        }catch(Exception){
            _response.error = "unhandled exception.";
            _response.data = null;
            _response.status = Status.Failed;
            return NotFound(_response);
        }
    }
    [HttpPost]
    public IActionResult newLanguageVersion(LanguageVersion languageVersion) {
        try{
            return Ok(_helper.newLanguageVersion(languageVersion));
        }catch(Exception){
            _response.error = "unhandled exception.";
            _response.data = null;
            _response.status = Status.Failed;
            return NotFound(_response);
        }
    }
    [HttpDelete("{id}")]
    public IActionResult deleteLanguageVersion(int id) {
        try{
            return Ok(_helper.deleteLanguageVersion(id));
        }catch(Exception){
            _response.error = "unhandled exception.";
            _response.data = null;
            _response.status = Status.Failed;
            return NotFound(_response);
        }
    }
    [HttpGet("{id}")]
    public IActionResult languageVersion(int id) {
        try{
            return Ok(_helper.getSingleLanguageVersion(id));
        }catch(Exception){
            _response.error = "unhandled exception.";
            _response.data = null;
            _response.status = Status.Failed;
            return NotFound(_response);
        }
    }
}