using Microsoft.AspNetCore.Mvc;
using request;
using models;
using response;
using context;
using helper;

[ApiController]
[Route("api/[controller]")]
public class UnitTestController : ControllerBase {
    private readonly ILogger<UnitTestController> _logger;
    private readonly TestPlatformResponse _response;
    public UnitTestHelper _helper { get ; set ; }
    public UnitTestController(ILogger<UnitTestController> logger, TestPlatformContext context) {
        TestPlatformResponse response = new TestPlatformResponse();
        _helper = new UnitTestHelper(response, context);
        _logger = logger;
        _response = response;
    }
    [HttpGet]
    public IActionResult unitTests() {
        try{
            return Ok(_helper.getAllUnitTests());
        }catch(Exception){
            _response.error = "unhandled exception.";
            _response.data = null;
            _response.status = Status.Failed;
            return NotFound(_response);
        }
    }
    [HttpPost]
    public IActionResult newUnitTest(AssignmentUnitTest unitTest) {
        try{
            return Ok(_helper.newUnitTest(unitTest));
        }catch(Exception){
            _response.error = "unhandled exception.";
            _response.data = null;
            _response.status = Status.Failed;
            return NotFound(_response);
        }
    }
    [HttpDelete("{id}")]
    public IActionResult deleteUnitTest(int id) {
        try{
            return Ok(_helper.deleteUnitTest(id));
        }catch(Exception){
            _response.error = "unhandled exception.";
            _response.data = null;
            _response.status = Status.Failed;
            return NotFound(_response);
        }
    }
    [HttpGet("{id}")]
    public IActionResult unitTest(int id) {
        try{
            return Ok(_helper.getSingleUnitTest(id));
        }catch(Exception){
            _response.error = "unhandled exception.";
            _response.data = null;
            _response.status = Status.Failed;
            return NotFound(_response);
        }
    }
}