using Microsoft.AspNetCore.Mvc;
using response;
using context;
using helper;
using models;
using request;

[ApiController]
[Route("api/[controller]")]
public class UserAssignmentController : ControllerBase {
    private readonly ILogger<UserAssignmentController> _logger;
    private readonly TestPlatformResponse _response;
    public UserAssignmentHelper _helper { get ; set ; }
    public UserAssignmentController(ILogger<UserAssignmentController> logger, TestPlatformContext context) {
        TestPlatformResponse response = new TestPlatformResponse();
        _helper = new UserAssignmentHelper(response, context);
        _logger = logger;
        _response = response;
    }
    [HttpGet]
    public IActionResult userAssignments() {
        try{
            return Ok(_helper.getAllUserAssignments());
        }catch(Exception){
            _response.error = "unhandled exception.";
            _response.data = null;
            _response.status = Status.Failed;
            return NotFound(_response);
        }
    }
    [HttpPost]
    public IActionResult newUserAssignment(UserAssignment userAssignment) {
        try{
            return Ok(_helper.newUserAssignment(userAssignment));
        }catch(Exception){
            _response.error = "unhandled exception.";
            _response.data = null;
            _response.status = Status.Failed;
            return NotFound(_response);
        }
    }
    [HttpDelete("{id}")]
    public IActionResult deleteUserAssignment(int id) {
        try{
            return Ok(_helper.deleteUserAssignment(id));
        }catch(Exception){
            _response.error = "unhandled exception.";
            _response.data = null;
            _response.status = Status.Failed;
            return NotFound(_response);
        }
    }
    [HttpPut("{id}")]
    public IActionResult updateUserAssignment(int id, UserAssignment userAssignment){
        try{
            return Ok(_helper.updateUserAssignment(id, userAssignment));
        }catch(Exception){
            _response.error = "unhandled exception.";
            _response.data = null;
            _response.status = Status.Failed;
            return NotFound(_response);
        }

    }
    [HttpGet("{id}")]
    public IActionResult userAssignment(int id) {
        try{
            return Ok(_helper.getSingleUserAssignment(id));
        }catch(Exception){
            _response.error = "unhandled exception.";
            _response.data = null;
            _response.status = Status.Failed;
            return NotFound(_response);
        }
    }
}