using Microsoft.EntityFrameworkCore;
using models;
namespace context{
    public class TestPlatformContext : DbContext {
        public TestPlatformContext(DbContextOptions<TestPlatformContext> options) : base(options) { }

        public DbSet<Test> tbl_test { get; set; }
        public DbSet<Assignment> tbl_assignment { get; set; }
        public DbSet<UserAssignment> tbl_user_assignment { get; set; }
        public DbSet<Attempt> tbl_attempt { get; set; }
        public DbSet<Language> tbl_language { get; set; }
        public DbSet<LanguageVersion> tbl_language_version { get; set; }
        public DbSet<Result> tbl_result { get; set; }
        public DbSet<AssignmentUnitTest> tbl_assignment_unit_test { get; set; }
        protected override void OnModelCreating(ModelBuilder modelBuilder) {
            modelBuilder.Entity<UserAssignment>()
                .HasOne(ua => ua.assignment)
                .WithMany(ua => ua.userAssignments)
                .HasForeignKey(ua => ua.assignmentID)
                .IsRequired();
            modelBuilder.Entity<UserAssignment>()
                .HasOne(ua => ua.test)
                .WithMany(ua => ua.userAssignments)
                .HasForeignKey(ua => ua.testID)
                .IsRequired();
            modelBuilder.Entity<Attempt>()
                .HasOne(ua => ua.languageVersion)
                .WithMany(lv => lv.attempts)
                .HasForeignKey(ua => ua.languageVersionID)
                .IsRequired();

            modelBuilder.Entity<Result>()
                .HasOne( r => r.attempt )
                .WithOne( a => a.result )
                .HasForeignKey<Attempt>( a => a.resultID )
                .IsRequired();
            
            modelBuilder.Entity<Attempt>()
                .HasOne( a => a.result )
                .WithOne( a => a.attempt )
                .HasForeignKey<Result>( a => a.attemptID )
                .IsRequired();

            modelBuilder.Entity<Attempt>()
                .HasOne( a => a.userAssignment )
                .WithMany( a => a.attempts )
                .HasForeignKey( a => a.userAssignmentID )
                .IsRequired();

            modelBuilder.Entity<LanguageVersion>()
                .HasOne( a => a.language )
                .WithMany( a => a.languageVersions )
                .HasForeignKey( a => a.languageID )
                .IsRequired();
            modelBuilder.Entity<AssignmentUnitTest>()
                .HasOne( a => a.language )
                .WithMany( a => a.unitTests )
                .HasForeignKey( a => a.languageID )
                .IsRequired();
            modelBuilder.Entity<AssignmentUnitTest>()
                .HasOne( a => a.assignment )
                .WithMany( a => a.unitTests )
                .HasForeignKey( a => a.assignmentID )
                .IsRequired();
            modelBuilder.Entity<Assignment>()
                .HasMany( a => a.languages )
                .WithMany( a => a.assignments );
        } 
    }
}