using Microsoft.EntityFrameworkCore;
using System.Linq;
namespace impl;
using models;
using context;
public class DBimpl
{
    private readonly TestPlatformContext _context;
    public DBimpl(TestPlatformContext context)
    {
        _context = context;
    }


    public List<Test>? getAllTest()
    {
        try
        {
            List<Test> tests = _context.tbl_test
            .Include(t => t.userAssignments)
            .ThenInclude(ua => ua.attempts)
            .ThenInclude(a => a.result)
            .Where(t => t.isDeleted == false).ToList();
            foreach (Test test in tests)
            {
                foreach (UserAssignment ua in test.userAssignments)
                {
                    ua.test = null;
                    foreach(Attempt a in ua.attempts){
                        a.userAssignment = null;
                        try{
                            a.result.attempt = null;
                        }catch(Exception){ //result may not be calculated.
                            a.result = null;
                        }
                    }
                }
            }
            return tests;
        }
        catch (Exception)
        {
            return null;
        }
    }
    public Test? getSingeTest(int testID)
    {
        try
        {
            Test test = _context.tbl_test
            .Include(t => t.userAssignments)
            .Single(t => t.id == testID && t.isDeleted == false);
            foreach (UserAssignment ua in test.userAssignments)
            {
                ua.test = null;
            }
            return test;
        }
        catch (Exception)
        {
            return null;
        }
    }
    public Test newTest(Test test)
    {
        test.isDeleted = false;
        try
        {
            _context.Add<Test>(test);
            _context.SaveChanges();
            return test;
        }
        catch (Exception)
        {
            return null;
        }
    }
    public bool deleteTest(Test test)
    {
        test.isDeleted = true;
        try
        {
            _context.Update<Test>(test);
            _context.SaveChanges();
            return true;
        }
        catch (Exception)
        {
            return false;
        }
    }

    public List<Assignment>? getAllAssignments()
    {
        try
        {
            List<Assignment> assignments = _context.tbl_assignment
            .Include(a => a.languages)
            .Include(a => a.unitTests)
            .Where(a => a.isDeleted == false).ToList();

            foreach (var ass in assignments)
            {
                foreach (var lang in ass.languages)
                {
                    lang.assignments = null;
                    lang.unitTests = null;
                }
                foreach (var unitTest in ass.unitTests)
                {
                    unitTest.assignment = null;
                    unitTest.language = null;
                }
            }
            return assignments;
        }
        catch (Exception)
        {
            return null;
        }

    }
    public Assignment? getSingleAssignment(int assignmentID)
    {
        try
        {
            Assignment assignment = _context.tbl_assignment
            .Include(assignment => assignment.unitTests.Where( l => l.isDeleted == false))
            .Include(assignment => assignment.languages.Where( l => l.isDeleted == false))
            .ThenInclude(language => language.languageVersions.Where( l=> l.isDeleted == false))
            .Single(assignment => assignment.id == assignmentID && assignment.isDeleted == false);
            foreach (Language lang in assignment.languages)
            {
                foreach (LanguageVersion lv in lang.languageVersions)
                {
                    lv.language = null;
                }
                lang.unitTests = null;
                lang.assignments = null;
                lang.unitTests = null;
            }
            foreach (AssignmentUnitTest ut in assignment.unitTests)
            {
                ut.language = null;
                ut.assignment = null;
            }

            return assignment;
        }
        catch (Exception)
        {
            return null;
        }
    }
    public bool updateAssignment(int id, Assignment assignment)
    {
        assignment.id = id;
        try
        {
            _context.Update(assignment);
            _context.SaveChanges();
            return true;
        }
        catch (Exception)
        {
            return false;
        }
    }
    public bool newAssignment(Assignment assignment)
    {
        assignment.isDeleted = false;
        try
        {
            _context.Add<Assignment>(assignment);
            _context.SaveChanges();
            return true;
        }
        catch (Exception)
        {
            return false;
        }
    }
    public bool deleteAssignment(Assignment assignment)
    {
        assignment.isDeleted = true;
        try
        {
            _context.Update<Assignment>(assignment);
            _context.SaveChanges();
            return true;
        }
        catch (Exception)
        {
            return false;
        }
    }
    public List<Language>? getAllLanguages()
    {
        try
        {
            List<Language> languages = _context.tbl_language
            .Where(l => l.isDeleted == false)
            .ToList();
            return languages;
        }
        catch (Exception)
        {
            return null;
        }

    }
    public Language? getSingleLanguage(int languageID)
    {
        try
        {
            Language language = _context.tbl_language
            .Single(language => language.id == languageID && language.isDeleted == false);
            return language;
        }
        catch (Exception)
        {
            return null;
        }

    }
    public bool newLanguage(Language language)
    {
        language.isDeleted = false;
        try
        {
            _context.Add<Language>(language);
            _context.SaveChanges();
            return true;
        }
        catch (Exception)
        {
            return false;
        }
    }
    public bool deleteLanguage(Language language)
    {
        language.isDeleted = true;
        try
        {
            _context.Update<Language>(language);
            _context.SaveChanges();
            return true;
        }
        catch (Exception)
        {
            return false;
        }
    }
    public List<LanguageVersion>? getAllLanguageVersions()
    {
        try
        {
            List<LanguageVersion> languageVersions = _context.tbl_language_version
            .Where(l => l.isDeleted == false)
            .ToList();
            return languageVersions;
        }
        catch (Exception)
        {
            return null;
        }

    }
    public LanguageVersion? getSingleLanguageVersion(int languageVersionID)
    {
        try
        {
            LanguageVersion languageVersion = _context.tbl_language_version
            .Single(l => l.id == languageVersionID && l.isDeleted == false);
            return languageVersion;
        }
        catch (Exception)
        {
            return null;
        }

    }
    public bool newLanguageVersion(LanguageVersion languageVersion)
    {
        languageVersion.isDeleted = false;
        try
        {
            _context.Add<LanguageVersion>(languageVersion);
            _context.SaveChanges();
            return true;
        }
        catch (Exception)
        {
            return false;
        }
    }
    public bool deleteLanguageVersion(LanguageVersion languageVersion)
    {
        languageVersion.isDeleted = true;
        try
        {
            _context.Update<LanguageVersion>(languageVersion);
            _context.SaveChanges();
            return true;
        }
        catch (Exception)
        {
            return false;
        }
    }

    public List<AssignmentUnitTest>? getAllUnitTests()
    {
        try
        {
            List<AssignmentUnitTest> unitTests = _context.tbl_assignment_unit_test
            .Where(aut => aut.isDeleted == false)
            .ToList();
            return unitTests;
        }
        catch (Exception)
        {
            return null;
        }

    }
    public AssignmentUnitTest? getSingleUnitTest(int unitTestID)
    {
        try
        {
            AssignmentUnitTest assignmentUnitTest = _context.tbl_assignment_unit_test
            .Single(l => l.id == unitTestID && l.isDeleted == false);
            return assignmentUnitTest;
        }
        catch (Exception)
        {
            return null;
        }

    }
    public bool newUnitTest(AssignmentUnitTest assignmentUnitTest)
    {
        assignmentUnitTest.isDeleted = false;
        try
        {
            _context.Add<AssignmentUnitTest>(assignmentUnitTest);
            _context.SaveChanges();
            return true;
        }
        catch (Exception)
        {
            return false;
        }
    }
    public bool deleteUnitTest(AssignmentUnitTest assignmentUnitTest)
    {
        assignmentUnitTest.isDeleted = true;
        try
        {
            _context.Update<AssignmentUnitTest>(assignmentUnitTest);
            _context.SaveChanges();
            return true;
        }
        catch (Exception)
        {
            return false;
        }
    }

    public List<UserAssignment>? getAllUserAssignments()
    {
        try
        {
            List<UserAssignment> userAssignments = _context.tbl_user_assignment
            .Include(ua => ua.assignment)
            .Include(ua => ua.test)
            .Where(a => a.isDeleted == false).ToList();
            foreach (UserAssignment ua in userAssignments)
            {
                ua.test.userAssignments = null;
            }
            return userAssignments;
        }
        catch (Exception)
        {
            return null;
        }

    }
    public UserAssignment? getSingleUserAssignment(int userAssignmentID)
    {
        try
        {
            UserAssignment userAssignment = _context.tbl_user_assignment
            .Include(ua => ua.assignment)
            .Single(ua => ua.id == userAssignmentID && ua.isDeleted == false);
            return userAssignment;
        }
        catch (Exception)
        {
            return null;
        }
    }
    public bool updateUserAssignment(int id, UserAssignment userAssignment)
    {
        userAssignment.id = id;
        try
        {
            _context.Update(userAssignment);
            _context.SaveChanges();
            return true;
        }
        catch (Exception)
        {
            return false;
        }
    }
    public UserAssignment? newUserAssignment(UserAssignment userAssignment)
    {
        userAssignment.isDeleted = false;
        try
        {
            _context.Add<UserAssignment>(userAssignment);
            _context.SaveChanges();
            return userAssignment;
        }
        catch (Exception)
        {
            return null;
        }
    }
    public bool deleteUserAssignment(UserAssignment userAssignment)
    {
        userAssignment.isDeleted = true;
        try
        {
            _context.Update<UserAssignment>(userAssignment);
            _context.SaveChanges();
            return true;
        }
        catch (Exception)
        {
            return false;
        }
    }
    public Attempt? newAttempt(Attempt attempt)
    {
        attempt.isDeleted = false;
        try
        {
            _context.Add<Attempt>(attempt);
            _context.SaveChanges();
            return attempt;
        }
        catch (Exception)
        {
            return null;
        }
    }
    public bool deleteAttempt(Attempt attempt)
    {
        attempt.isDeleted = true;
        try
        {
            _context.Update<Attempt>(attempt);
            _context.SaveChanges();
            return true;
        }
        catch (Exception)
        {
            return false;
        }
    }
    public Attempt? getSingleAttempt(int attemptId)
    {
        try
        {
            Attempt attempt = _context.tbl_attempt
            .Single(a => a.id == attemptId && a.isDeleted == false);
            return attempt;
        }
        catch (Exception)
        {
            return null;
        }
    }

    public bool deleteResult(Result result)
    {
        result.isDeleted = true;
        try
        {
            _context.Update<Result>(result);
            _context.SaveChanges();
            return true;
        }
        catch (Exception)
        {
            return false;
        }
    }
    public Result? getSingleResult(int resultID)
    {
        try
        {
            Result result = _context.tbl_result
            .Single(r => r.id == resultID && r.isDeleted == false);
            return result;
        }
        catch (Exception)
        {
            return null;
        }
    }
    public List<Result>? getAllResults()
    {
        try
        {
            List<Result> results = _context.tbl_result
            .Where(r => r.isDeleted == false).ToList();
            return results;
        }
        catch (Exception)
        {
            return null;
        }
    }
}