namespace executionenv;
using Microsoft.EntityFrameworkCore;
using Hangfire;
using CliWrap;
using CliWrap.Buffered;
using context;
using models;
using Microsoft.AspNetCore.SignalR.Client;
using System.Text.Json;

public class JavascriptEnv : IEnviorment
{
    private TestPlatformContext _context;
    public JavascriptEnv(TestPlatformContext context)
    {
        _context = context;
    }
    public async Task<string> exec(int attemptID, string connectionId) {
        Attempt attempt = _context.tbl_attempt
            .Include( a => a.userAssignment)
            .Include( a => a.languageVersion)
            .Single(attempt => attempt.id == attemptID);
        AssignmentUnitTest uat = _context.tbl_assignment_unit_test
            .Single( ut => ut.assignmentID == attempt.userAssignment.assignmentID 
                && ut.languageID == attempt.languageVersion.languageID 
                && ut.isDeleted == false
            );
        await Cli.Wrap("/bin/mkdir").WithArguments("/tmp/jsenv").WithValidation(0).ExecuteBufferedAsync();
        await Cli.Wrap("/bin/touch").WithArguments("/tmp/jsenv/testClass.js").WithValidation(0).ExecuteBufferedAsync();
        await Cli.Wrap("/bin/touch").WithArguments("/tmp/jsenv/testClass.test.js").WithValidation(0).ExecuteBufferedAsync();
        await Cli.Wrap("/bin/touch").WithArguments("/tmp/jsenv/package.json").WithValidation(0).ExecuteBufferedAsync();

        string targetPath = "/tmp/jsenv/testClass.js";
        using (StreamWriter writer = new StreamWriter(targetPath)) {
            writer.WriteLine(uat.compileTest);
            writer.Write(attempt.code);
            writer.WriteLine("module.exports = { " + uat.unitTestMethod +" };");
        }
        var execResult = await Cli.Wrap("/bin/node").WithArguments(targetPath).WithValidation(0).ExecuteBufferedAsync();
        //await Cli.Wrap("/bin/rm").WithArguments(targetPath).ExecuteAsync();


        //save result to Database based off execution.
        Result result = new Result();
        if (execResult.ExitCode == 0) {
            string unitTest = await execUnitTest(uat);
            string unitTestSecret = await execSecretUnitTest(uat);
            result.complete = Complete.ALL_UNIT_TESTS_FAILED;
            if(unitTest.Contains("failed") && unitTest.Contains("passed")){
                result.complete = Complete.PARTLY_CORRECT;
            }else if(!unitTest.Contains("fail") && !unitTest.Contains("not found")) {
                result.complete = Complete.HIDDEN_TESTS_FAILED;
                if(!unitTestSecret.Contains("fail")){
                    result.complete = Complete.CORRECT;
                }
            }
            result.output = unitTest;
        }
        else {
            result.complete = Complete.COMPILE_FAILED;
            result.output = execResult.StandardError;
        }

        result.attemptID = attempt.id;
        result.runtime = execResult.RunTime.Milliseconds;
        _context.tbl_result.Add(result);
        _context.SaveChanges();

        Result cpyResult = new Result();
        cpyResult.id = result.id;
        cpyResult.attemptID = result.attemptID;
        cpyResult.complete = result.complete;
        cpyResult.DateExec = result.DateExec;
        cpyResult.output = result.output;
        cpyResult.runtime = result.runtime;
        cpyResult.isDeleted = false;
        cpyResult.attempt = null;

        callClient(cpyResult, connectionId);


        return "";
    }

    public async Task<string> execUnitTest(AssignmentUnitTest unitTest)
    {
        string targetPath = "/tmp/jsenv/package.json";
        string packageContent = "{\"name\": \"jsenv\", \"version\": \"1.0.0\", \"description\": \"\", \"main\": \"test.js\", \"scripts\": { \"test\": \"jest\" }, \"devDependencies\": { \"jest\": \"^27.0.6\" } } ";
        using (StreamWriter writer = new StreamWriter(targetPath))
        {
            writer.Write(packageContent);
        }
        using (StreamWriter writer = new StreamWriter("/tmp/jsenv/testClass.test.js"))
        {
            writer.Write(unitTest.unitTest);
        }
        var result = await Cli.Wrap("npm").WithArguments("test").WithWorkingDirectory("/tmp/jsenv").WithValidation(0).ExecuteBufferedAsync();
        return result.StandardError;
    }
    public async Task<string> execSecretUnitTest(AssignmentUnitTest unitTest)
    {
        string targetPath = "/tmp/jsenv/package.json";
        string packageContent = "{\"name\": \"jsenv\", \"version\": \"1.0.0\", \"description\": \"\", \"main\": \"test.js\", \"scripts\": { \"test\": \"jest\" }, \"devDependencies\": { \"jest\": \"^27.0.6\" } } ";
        using (StreamWriter writer = new StreamWriter(targetPath))
        {
            writer.Write(packageContent);
        }
        using (StreamWriter writer = new StreamWriter("/tmp/jsenv/testClass.test.js"))
        {
            writer.Write(unitTest.secretUnitTest);
        }
        var result = await Cli.Wrap("npm").WithArguments("test").WithWorkingDirectory("/tmp/jsenv").WithValidation(0).ExecuteBufferedAsync();
        return result.StandardError;
    }

    public async Task<string> callClient(Result result, string connectionId){
        HubConnection connection = connection = new HubConnectionBuilder().WithUrl("http://localhost:5291/resultHub").Build();
        try{
            await connection.StartAsync();
            await connection.InvokeAsync("SendResult", (JsonSerializer.Serialize<Result>(result)), connectionId);
        }catch (Exception ex){

        } 



        return "";
    }

}