import React from 'react'
import { useState, useEffect, useRef} from 'react';
import AdminButton from './AdminButton';
import AdminTextbox from './AdminTextbox';
import AdminInput from './AdminInput';
import AdminCheckbox from './AdminCheckbox';
import Select from '../Select';
import AdminInputNum from './AdminInputNum';
import utils from '../../utils/AssignmentUtils'
import AdminMultipleCheckbox from './AdminMultipleCheckbox';

function AdminAssignmentFields( { assignment, handler, type } ) {
    const [response, setResponse] = useState();
    const [languages, setLanguages] = useState([]);

    useEffect(()=> { 
        fetch('http://localhost:5291/api/Language')
            .then((res) => res.json())
            .then((data) => {
                console.log(data);
            	setLanguages(data.data); 
			})
	} ,[])
    
    function updateField(val, prop){
        if(prop=="released"){
            const tmp = {...assignment};
            if(val){
                tmp[`${prop}`] = new Date().toJSON();
                handler(tmp);
            }else{
                tmp[`${prop}`] = new Date(0).toJSON();
                handler(tmp);
            }
        }else{
            const tmp = {...assignment};
            tmp[`${prop}`] = val;
            handler(tmp);

        }
    }
    function responseOutput() {
        if(response.status == 2){
            return(<div>RESPONSE ERROR: {response.error}</div>)
        }else if(response.status == 1){
            return(<div>Successfully handled action</div>)
        }else{
            return(<div>fatefully rejected response</div>)
        }
    }
    async function handleNew(action) {
        switch(action){
            case "delete":
                await utils.deleteAssignment(assignment).then(
                    (resp) => setResponse(resp)
                );
            break;
            case "update":
                await utils.putAssignment(assignment).then(
                    (resp) => setResponse(resp)
                );
            break;
            case "upload":
                await utils.postAssignment(assignment).then(
                    (resp) => setResponse(resp)
                );
            break;
            default:
            break;
        }
    }
    async function setList(newList, type){
        const newAssignment = {...assignment};
        if(type == "language"){
            newAssignment.languages = newList;
            handler(newAssignment)
        }
    }


  return (
    <div className='admin-field'>
        <div className='admin-text'>
            Name
        </div>
        <AdminInput inputvalue={assignment.name} fieldToUpdate="name" handler={updateField}/>
        <div className='admin-text'>
            Description
        </div>
        <AdminTextbox value={assignment.description} fieldToUpdate="description" handler={updateField}/>
        <div className='admin-text'>
            Difficulty
        </div>
        <AdminInputNum inputvalue={assignment.difficulty} fieldToUpdate="difficulty" handler={updateField} />
        <div className='admin-text'>
            Publish
        </div>
        <AdminCheckbox inputname={"published"} 
            inputvalue={assignment.released!=undefined||null?new Date(assignment.released).getTime() > new Date(0).getTime():false} 
            handler={updateField} fieldToUpdate={"released"}/>
        <div className='admin-text'>
            available languages/versions
        </div>
        { languages != undefined && <AdminMultipleCheckbox listAvailable={languages} list={assignment.languages} handler={setList} type="language"/>}
        <div className='btn-container'>
            {type=='new'&&<AdminButton name="upload" handler={handleNew}/>}
            {type=='update'&&<AdminButton name="update" handler={handleNew}/>}
            {type=='update'&&<AdminButton name="delete" color="red" handler={handleNew}/>}
        </div>
        {response != null && responseOutput()}
    
    </div>
  )
}

export default AdminAssignmentFields
