import React, { useState } from 'react'
import AdminTextbox from './AdminTextbox'
import AdminInput from './AdminInput';
import utils from '../../utils/UnitTestUtils';
import AdminButton from './AdminButton';
import AdminSelect from './AdminSelect';

function AdminUnitTestFields({ unitTest, languages, assignments, handler, type }) {
    const [response, setResponse] = useState();

    function updateField(val, prop){
		const tmp = {...unitTest};
		if(prop=="assignment"){
            tmp.assignmentID = parseInt(val);
            tmp[`${prop}`] = assignments.find( x => parseInt(val) == x.id);
        }else if(prop == "language"){
            tmp.languageID = parseInt(val);
            tmp[`${prop}`] = languages.find( x => parseInt(val) == x.id);
        }else{
            tmp[`${prop}`] = val;
        }
		handler(tmp);
    }
    async function handleNew(action) {
        switch(action){
            case "delete":
                await utils.deleteUnitTest(unitTest).then(
                    (resp) => setResponse(resp)
                );
            break;
            case "upload":
                await utils.postUnitTest(unitTest).then(
                    (resp) => setResponse(resp)
                );
            break;
            default:
				setResponse(undefined)
            break;
        }
    }
    function responseOutput() {
        if(response.status == 2){
            return(<div>RESPONSE ERROR: {response.error}</div>)
        }else if(response.status == 1){
            return(<div>Successfully handled action</div>)
        }else{
            return(<div>fatefully rejected response</div>)
        }
    }
    function firstRenderSelect(){
        if(unitTest.assignmentID == 999999 && assignments.length > 0){
            updateField(assignments[0].id, "assignment")
        }
        if(unitTest.languageID == 999999 && languages.length > 0){
            updateField(languages[0].id, "language")
        }
    }
  return (
    <div className='admin-field'>
        {type == 'new' && <div className='admin-text'> Assignment & Language </div>}
        {type == 'new' && firstRenderSelect()}
		{type == 'new' && <AdminSelect list={assignments} handler={updateField} fieldToUpdate={"assignment"} />}
		{type == 'new' && <AdminSelect list={languages} handler={updateField} fieldToUpdate={"language"} />}
        <div className='admin-text'>
            name of methods seperated by comma and space without ().
        </div>
        <AdminInput inputvalue={unitTest.unitTestMethod} fieldToUpdate="unitTestMethod" handler={updateField}/>
        <div className='admin-text'>
            visible unit test
        </div>
        <AdminTextbox value={unitTest.unitTest} fieldToUpdate="unitTest" handler={updateField}/>
        <div className='admin-text'>
            secret unit test
        </div>
        <AdminTextbox value={unitTest.secretUnitTest} fieldToUpdate="secretUnitTest" handler={updateField}/>

        <div className='admin-text'>
            Start Code
        </div>
        <AdminTextbox value={unitTest.startCode} fieldToUpdate="startCode" handler={updateField}/>

        <div className='admin-text'>
            Hard Compile Test
        </div>
        <AdminTextbox value={unitTest.compileTest} fieldToUpdate="compileTest" handler={updateField}/>

        <div className='btn-container'>
            {type=='new'&&<AdminButton name="upload" handler={handleNew}/>}
            {type=='update'&&<AdminButton name="delete" color="red" handler={handleNew}/>}
        </div>
        {response != null && responseOutput()}
    </div>
  )
}
export default AdminUnitTestFields
