import React, { useEffect, useState } from 'react'
import FilterCheckbox from './FilterCheckbox'
import FilterIcon from '../resources/Filter.svg'
import FilterArrow from '../resources/filterArrow.svg'
import Utils from '../utils/Utils'

function Filter({ orphanTest, handler, assignments }) {
    const[difficulties, setDifficulties] = useState([0,1,2,3])
    const[completes, setCompletes] = useState([0,1,2,3])
    const[order, setOrder] = useState("");

    function handleFilter(value, type, element){
        var newDifficulties = [...difficulties];
        var newCompletes = [...completes];
        var orderSync = order
        if(type=="difficulty"){
            if(!value){
                newDifficulties = newDifficulties.filter(e => e != element)
                setDifficulties(newDifficulties);
            }else{
                newDifficulties.push(element)
                setDifficulties(newDifficulties)
            }
        }
        if(type=="completes"){
            if(!value){
                newCompletes = newCompletes.filter(e => e != element)
                setCompletes(newCompletes);
            }else{
                newCompletes.push(element)
                setCompletes(newCompletes)
            }
        }
        if(type == "sort"){
            setOrder(value)
            orderSync = value
        }
        var newFilteredList = [...assignments]
        newFilteredList = newFilteredList.filter( a => newDifficulties.includes(a.difficulty))
        newFilteredList = newFilteredList.filter( a => newCompletes.includes(Utils.mapProgress(a,orphanTest)))
        if(orderSync=="difficulty"){
            newFilteredList.sort((a,b) =>{
                return a.difficulty - b.difficulty
            })
        }else if(orderSync == "status"){
            newFilteredList.sort((a,b) => {
                return Utils.mapProgress(a,orphanTest) - Utils.mapProgress(b,orphanTest)
            })
        }
        handler(newFilteredList)
    }
  return (
    <div className='filter-container'>
        <div className='filter-section'>
            <div className='filter-section-header'>
                Difficulty
            </div>
            <div className='filter-section'>
                <div className='filter-element'>
                    <FilterCheckbox boxValue={0} checkValue={difficulties.includes(0)} handler={handleFilter} type="difficulty"/>
                    <div className='filter-info'>
                        Basic
                    </div>
                </div>
                <div className='filter-element'>
                    <FilterCheckbox boxValue={1} checkValue={difficulties.includes(1)} handler={handleFilter} type="difficulty"/>
                    <div className='filter-info'>
                        Intermediate
                    </div>
                </div>
                <div className='filter-element'>
                    <FilterCheckbox boxValue={2} checkValue={difficulties.includes(2)} handler={handleFilter} type="difficulty"/>
                    <div className='filter-info'>
                        Advanced
                    </div>
                </div>
                <div className='filter-element'>
                    <FilterCheckbox boxValue={3} checkValue={difficulties.includes(3)} handler={handleFilter} type="difficulty"/>
                    <div className='filter-info'>
                        Expert
                    </div>
                </div>
            </div>
            <div className='filter-section-header'>
                Status
            </div>
            <div className='filter-section'>
                <div className='filter-element'>
                    <FilterCheckbox boxValue={0} checkValue={completes.includes(0)} handler={handleFilter} type="completes"/>
                    <div className='filter-info'>
                        Not Started
                    </div>
                </div>
                <div className='filter-element'>
                    <FilterCheckbox boxValue={1} checkValue={completes.includes(1)} handler={handleFilter} type="completes"/>
                    <div className='filter-info'>
                        Attempted
                    </div>
                </div>
                <div className='filter-element'>
                    <FilterCheckbox boxValue={2} checkValue={completes.includes(2)} handler={handleFilter} type="completes"/>
                    <div className='filter-info'>
                        Progressing
                    </div>
                </div>
                <div className='filter-element'>
                    <FilterCheckbox boxValue={3} checkValue={completes.includes(3)} handler={handleFilter} type="completes"/>
                    <div className='filter-info'>
                        Completed 
                    </div>
                </div>
            </div>
            <div className='filter-section-header'>
                Sort by
            </div>
            <div className='filter-section'>
                <div className='filter-element' onClick={() => handleFilter("difficulty", "sort", "")}>
                    <img src={FilterIcon} className='filter-img' />
                    <div className='filter-sort' >
                       Difficulty 
                    </div>
                    {order == "difficulty" && <img src={FilterArrow} className='filter-img-arrow' />}
                </div>
                <div className='filter-element' onClick={() => handleFilter("status", "sort", "")}>
                    <img src={FilterIcon} className='filter-img' />
                    <div className='filter-sort' >
                        Status
                    </div>
                    {order == "status" && <img src={FilterArrow} className='filter-img-arrow' />}
                </div>
            </div>
        </div>
    </div>
  )
}

export default Filter