import React, { useState, useEffect } from 'react'
import { useNavigate } from 'react-router-dom';
import CodeEditorWindow from "../components/CodeEditorWindow.js";
import { useLocation } from 'react-router-dom';
import Button from '../components/ButtonAssignment.js';
import Select from '../components/Select.js';
import AttemptUtils from '../utils/AttemptUtils.js';
import UserUtils from '../utils/UserUtils.js';
import Console from '../components/Console.js';
import * as signalR from '@microsoft/signalr';

function AssignmentView({ name }) {
	const [assignment, setAssignment] = useState();
	const [language, setLanguage] = useState({name:'javascript', id:2});
	const [attempt, setAttempt] = useState({userAssignmentID:9999, code:"", result:null, resultID:0, date:new Date().getTime()});
	const [result, setResult] = useState({id:1, output:"", runtime: 0 });
	const [test, setTest] = useState();
	const [userAssignment, setUserAssignment] = useState();
	const [tab, setTab] = useState("description");
	const [buttonName, setButtonName] = useState("run");
	const [connection, setConnection] = useState(null);
	const location = useLocation();
	const navigate = useNavigate();

    useEffect(() => {
        const newConnection = new signalR.HubConnectionBuilder()
            .withUrl('http://localhost:5291/resultHub')
            .withAutomaticReconnect()
            .build();
        setConnection(newConnection);
    }, []);

	useEffect(() => {
        if (connection) {
            connection.start()
                .then(result => {

                    connection.on('result', resultMessage => {
						console.log(JSON.parse(resultMessage));
						setResult(JSON.parse(resultMessage))
						setTab("output")
						setButtonName("run")
                    });

                    connection.on('status', status => {
						console.log("status:")
						console.log(status);
                    });
                })
                .catch(e => console.log('Connection failed: ', e));
        }
    }, [connection]);

	const setAttemptCode = (code, event) => {
		const newAttempt = {...attempt}
		newAttempt.code = code;
		setAttempt(newAttempt);
	}
	async function handleButton(type){
		if(type=="run"){
			setButtonName("loading")
			const resetResult = {...result}
			resetResult.output = "";
			resetResult.runtime = 0;
			resetResult.complete = undefined;
			setResult()
			var testSync = {};
			var userAssignmentSync = {};
			if(test != undefined && test != null){
				testSync = {...test};
			}
			if(userAssignment != undefined || userAssignment != null){
				userAssignmentSync = {...userAssignment};
			}
			if(Object.keys(testSync).length == 0 && await !(await UserUtils.checkIfUserHasTest(name)).valueOf()){
				//if the user does not own a orphan test, create one.
				const resp = await UserUtils.postOrphanTest(name);
				testSync = resp.data;
				setTest(testSync)
			}else if(Object.keys(testSync).length == 0){
				//get the test and set it.
				const resp = await UserUtils.fetchOrphanTestByUserName(name);
				//needs to be updated. get the single orphan test.
				testSync = resp.data[0];
				setTest(testSync);
			}
			if(Object.keys(userAssignmentSync).length == 0 && await !(await UserUtils.checkIfAssignmentInOrphan(name, assignment.id)).valueOf()){
				//if no userassignment
				const resp = await UserUtils.postUserAssignment(testSync.id, assignment.id);
				userAssignmentSync = resp.data;
				setUserAssignment(userAssignment);
			}else if(Object.keys(userAssignmentSync).length == 0){
				const resp = await UserUtils.fetchUserAssignmentByTest(testSync.id);
				userAssignmentSync = resp.data.find(x => x.assignmentID == assignment.id && x.testID == testSync.id);
				debugger
				setUserAssignment(userAssignmentSync);
			}
			const attemptArgs = {...attempt}
			attemptArgs.userAssignmentID = userAssignmentSync.id;
			attemptArgs.languageVersionID = 8;
			if (connection) {
				try {
					await connection.send('RecieveAttempt', JSON.stringify(AttemptUtils.convertAttempt(attemptArgs)));
				}
				catch(e) {
					console.log(e);
				}
			}
		}else if(type=="save") {
			//check if there is any attempts
			//if no attempt exit
			//if attempt set last attemptID to submissionID
			//redirect to tests
			navigate("/test")
		}
	}
	useEffect(() => {
		if(location.state != null){
		fetch('http://localhost:5291/api/assignment/' + location.state.id)
			.then((res) => res.json())
			.then((data) => {
				if (data.data) {
					setAssignment(data.data);
					console.log(data.data);
				}
			})

		}
	}, [])

	if (location.state != null) {
		return (
			<div className="assignment-container">
				<div className='assignment-header-container'>
					<div className='assignment-header'>
						<h1>
							{location.state.name}
						</h1>
					</div>
					<div className='assignment-button-container'>
						<Select list={assignment!=undefined?assignment.languages:[]}/>
						<Select list={assignment!=undefined&&assignment.languages.length>0 ? assignment.languages.find(x => x.id == language.id).languageVersions : []}/>
						<Button name="Save" type="save" handler={handleButton}/>
						<Button name="Run" type={buttonName} handler={handleButton}/>
					</div>
				</div>
				<div className='assignment-split'>
					<div className='assingnment-tab-container'>
						<div className='assignment-tabs'>
							<p className={tab=="description"?'assignment-tab-selected':'assignment-tab'} onClick={() => setTab("description")}>Description</p>
							<p className={tab=="output"?'assignment-tab-selected':'assignment-tab'} onClick={() => setTab("output")}>Output</p>
							<p className={tab=="history"?'assignment-tab-selected':'assignment-tab'} onClick={() => setTab("history")}>History</p>
						</div>
						<div className={tab=="description"?'assignment-tab-content':'assignment-tab-content-2'}>
								{tab=='description' && <pre>{location.state.description}</pre>}
								{tab=='output' && <Console result={result!=undefined?result:{}}/>}
						</div>
					</div>


					<div className='assignment-editor-container'>
						<div className='assignment-editor-main'>
							<div className='assignment-editor-header'>
								Code
							</div>
							<CodeEditorWindow value={assignment != undefined ? assignment.unitTests.find(x => x.languageID == language.id).startCode:''} width="100%" height="55vh" onChange={setAttemptCode} />
						</div>
						<div className='assignment-editor-test'>
							<div className='assignment-editor-header'>
								Unit test
							</div>
							<CodeEditorWindow 
								language="javascript"
								width="100%"
								height="23vh"
								value={assignment!=undefined?assignment.unitTests.find(x => x.languageID == language.id).unitTest:'unittest not found for this assignment and '+language.name} />
						</div>
					</div>
				</div>
			</div>
		)
	}else{
		return(
			<div>
				hei am	
			</div>
		)
	}
}
/*

*/

export default AssignmentView;
